@@ -1,5 +1,5 @@
 #  Pyrogram - Telegram MTProto API Client Library for Python
-#  Copyright (C) 2017-2021 Dan <https://github.com/delivrance>
+#  Copyright (C) 2017-present Dan <https://github.com/delivrance>
 #
 #  This file is part of Pyrogram.
 #
@@ -79,10 +79,10 @@ class RestrictChatMember(Scaffold):
                     until_date=until_date,
                     send_messages=True if not permissions.can_send_messages else None,
                     send_media=True if not permissions.can_send_media_messages else None,
-                    send_stickers=True if not permissions.can_send_stickers else None,
-                    send_gifs=True if not permissions.can_send_animations else None,
-                    send_games=True if not permissions.can_send_games else None,
-                    send_inline=True if not permissions.can_use_inline_bots else None,
+                    send_stickers=True if not permissions.can_send_other_messages else None,
+                    send_gifs=True if not permissions.can_send_other_messages else None,
+                    send_games=True if not permissions.can_send_other_messages else None,
+                    send_inline=True if not permissions.can_send_other_messages else None,
                     embed_links=True if not permissions.can_add_web_page_previews else None,
                     send_polls=True if not permissions.can_send_polls else None,
                     change_info=True if not permissions.can_change_info else None,
