@@ -1,5 +1,5 @@
 #  Pyrogram - Telegram MTProto API Client Library for Python
-#  Copyright (C) 2017-2021 Dan <https://github.com/delivrance>
+#  Copyright (C) 2017-present Dan <https://github.com/delivrance>
 #
 #  This file is part of Pyrogram.
 #
@@ -36,7 +36,8 @@ class MemoryStorage(SQLiteStorage):
 
         if self.name != ":memory:":
             dc_id, test_mode, auth_key, user_id, is_bot = struct.unpack(
-                self.SESSION_STRING_FORMAT,
+                (self.SESSION_STRING_FORMAT if len(self.name) == MemoryStorage.SESSION_STRING_SIZE else
+                 self.SESSION_STRING_FORMAT_64),
                 base64.urlsafe_b64decode(
                     self.name + "=" * (-len(self.name) % 4)
                 )
@@ -50,4 +51,4 @@ class MemoryStorage(SQLiteStorage):
             await self.date(0)
 
     async def delete(self):
-        raise NotImplementedError
+        pass
